/************************************************************
UDP communication example for CodeVisionAVR V3.36+
(C) Copyright 2018-2019 Pavel Haiduc, HP InfoTech S.R.L.
http://www.hpinfotech.ro

Hardware: Arduino Uno ATmega328 board + ENC28J60 shield.

The connections to the ENC28J60 chip are specified in the
Project|Configure|C Compiler|Libraries|ENC28J60 menu:

ENC28J60 ATmega328P
SI       PORTB.3 MOSI
SO       PORTB.4 MISO
SCK      PORTB.5 SCK
/CS      PORTB.2

Use the CodeVisionAVR Terminal @ 9600 8N1 to connect to
the board.

Open the Windows Control Panel > Network and Internet >
Network and Sharing Center > Change Adapter Settings.
Double click on the Local Area Connection.
Select Properties in the Status window.
Select Internet Protocol Version 4 > Properties.
Configure it to use the Static IP: 192.168.1.100,
Subnet mask: 255.255.255.0, Gateway: 192.168.1.250.

Connect the ENC28J60 shield to your PC's Ethernet Interface.
To send UDP messages from the PC you can use the
"Yet Another Terminal" (YAT) program from:
https://sourceforge.net/projects/y-a-terminal/
It must be configured to send text messages to
IP: 192.168.1.10 port: 10000.

To each message received from YAT, the Arduino Uno
with ENC28J60 will respond with: "Hello from CodeVisionAVR :-)"
and the message number.

The code size exceeds the limit of the Evaluation version.
The precompiled .hex file from the
c:\cvavr\Examples\AVR\ENC28J60\UDP\Release\Exe
directory can be uploaded to the Arduino Uno using
the Tools|Upload to Arduino menu.
************************************************************/

#include <EtherCard.h>
#include <string.h>
#include <stdio.h>
#include <iobits.h>
#include <delay.h>
#include <io.h>

uint8_t mymac[] = { 0xDE, 0xDD, 0xDD, 0x00, 0x00, 0x01 }; // specify the MAC of the ENC28J60 shield
uint8_t myip[] = { 192, 168, 1, 10 }; // specify the IP of the ENC28J60 shield
uint8_t gwip[] = { 192, 168, 1, 250 }; // specify the IP of the gateway
uint8_t destip[] = {192, 168, 1, 100 }; // specify the IP of the PC

uint8_t ether_buffer[800];

// Timer1 overflow interrupt service routine called every 10ms
interrupt [TIM1_OVF] void timer1_ovf_isr(void)
{
	// Reinitialize Timer1 value
	TCNT1H = 0xB1E0 >> 8;
	TCNT1L = 0xB1E0 & 0xff;

	// used by EtherCard to count time
	ether_timerproc();
}

uint8_t udp_data[508 + 1];
uint16_t cnt = 0;

// callback that prints received packets to the serial port
// and sends a text using UDP
void udp_serial_print(uint16_t dest_port, uint8_t *ip, uint16_t src_port, char *data, uint16_t len)
{

	printf(">>> Source IP:");
	ether_printIp(ip);
	printf(", source port: %u, dest port: %u, length: %u\r\n"
		   "Received data: %s\r\n\r\n", src_port, dest_port, len, data);

	printf("Sending response to IP:");
	ether_printIp(ip);
	printf("\r\n");

	len = sprintf(udp_data, "Message #:%u, Hello from CodeVisionAVR :-)", ++cnt);
	ether_sendUdp(udp_data, len, 10000, destip, 10000);
	printf("Transmitted data: %s\r\nlength %u\r\n\r\n", udp_data, len);
}

void main(void)
{
	uint16_t len;

	// Timer/Counter 1 initialization
	// Clock source: System Clock
	// Clock value: 2000.000 kHz
	// Mode: Normal top=0xFFFF
	// OC1A output: Disconnected
	// OC1B output: Disconnected
	// Noise Canceler: Off
	// Input Capture on Falling Edge
	// Timer Period: 10 ms
	TCCR1A = (0 << COM1A1) | (0 << COM1A0) | (0 << COM1B1) | (0 << COM1B0) | (0 << WGM11) | (0 << WGM10);
	TCCR1B = (0 << ICNC1) | (0 << ICES1) | (0 << WGM13) | (0 << WGM12) | (0 << CS12) | (1 << CS11) | (0 << CS10);
	TCNT1H = 0xB1;
	TCNT1L = 0xE0;

	// Timer1 Overflow Interrupt: On
	// Input Capture Interrupt: Off
	// Compare A Match Interrupt: Off
	// Compare B Match Interrupt: Off
	TIMSK1 = (0 << ICIE1) | (0 << OCIE1B) | (0 << OCIE1A) | (1 << TOIE1);

	// USART initialization
	// Communication Parameters: 8 Data, 1 Stop, No Parity
	// USART Receiver: Off
	// USART Transmitter: On
	// USART Mode: Asynchronous
	// USART Baud Rate: 9600 (Double Speed Mode)
	UCSR0A = (0 << RXC0) | (0 << TXC0) | (0 << UDRE0) | (0 << FE0) | (0 << DOR0) | (0 << UPE0) | (1 << U2X0) | (0 << MPCM0);
	UCSR0B = (0 << RXCIE0) | (0 << TXCIE0) | (0 << UDRIE0) | (0 << RXEN0) | (1 << TXEN0) | (0 << UCSZ02) | (0 << RXB80) | (0 << TXB80);
	UCSR0C = (0 << UMSEL0) | (0 << UPM01) | (0 << UPM00) | (0 << USBS0) | (1 << UCSZ01) | (1 << UCSZ00) | (0 << UCPOL0);
	UBRR0H = 0x00;
	UBRR0L = 0xCF;

	// Set TXD0 (PORTD bit 1) as output in high state
	DDRD.1 = 1;
	PORTD.1 = 1;

    #asm("sei")

	printf("CodeVisionAVR UDP TX/RX demo\r\n");

	if (ether_begin(sizeof ether_buffer, mymac) == 0)
		printf("Failed to access the Ethernet controller\r\n");
	else
		{
		if (ether_staticSetup(myip, gwip, 0, 0))
			{
			printf("Static IP setup OK\r\nENC28J60 IP:");
			ether_printIp(ether.myip);
			printf("\r\nGateway IP: ");
			ether_printIp(ether.gwip);
			printf("\r\n\r\n");

			//register udp_serial_print() to port 10000
			ether_udpServerListenOnPort(&udp_serial_print, 10000);

			while (1)
				{
				len = enc28j60_packetReceive(); // receive packet
				ether_packetLoop(len);
				}
			}
		else
			printf("Error setting the static IP\r\n");
		}


}
